// DlgRebar.cpp


#include "stdafx.h"
#include "SimuRoute.h"
#include "DlgRebar.h"

#include "GraphView.h"

#include <math.h>


CDlgRebar e_dlgRebar;


const int minSpeed =    1;
const int maxSpeed =   40;

const int minSkip  =   0;
const int maxSkip  = 160;



BEGIN_MESSAGE_MAP(CDlgRebar, CDialogBar)
	ON_WM_HSCROLL()
END_MESSAGE_MAP()


// Initialize
void CDlgRebar::Init()
{
	m_sldSpeed.SubclassDlgItem(IDCSLD_SPEED, this);
	m_sldSpeed.SetRange(minSpeed, maxSpeed);
	
	m_sldSkip .SubclassDlgItem(IDCSLD_SKIP, this);
	m_sldSkip .SetRange(minSkip, maxSkip);
}


// Slider
void CDlgRebar::OnHScroll(UINT, UINT, CScrollBar*)
{
	CGraphView *const pView = app.GetActiveView();
	if (pView) {
		pView->SimulUpdateTimer(GetSpeed());
		pView->m_iSimulSkip = GetSkip();
		
		UpdateSkipLabel();
	}
}


// Update the "skip frames" numeric indicator
void CDlgRebar::UpdateSkipLabel()
{
	SetDlgItemInt(IDCLBL_SKIP, (UINT)GetSkip());
}


// Get the speed (logarithmic)
int CDlgRebar::GetSpeed()
{
	return (int)exp((double)(maxSpeed+minSpeed - m_sldSpeed.GetPos()) * 0.2);
}

// Set the speed (logarithmic)
void CDlgRebar::SetSpeed(int iSpeed)
{
	m_sldSpeed.EnableWindow(iSpeed>=0);
	if (iSpeed >= 0)
		m_sldSpeed.SetPos(maxSpeed+minSpeed - (int)(5.0 * log((double)iSpeed)));
}


// Get the skip rate
int CDlgRebar::GetSkip()
{
	return (int)floor(0.5 + pow(10.0, (double)m_sldSkip.GetPos() * 0.025));
}

// Set the skip rate
void CDlgRebar::SetSkip(int iSkip)
{
	m_sldSkip.EnableWindow(iSkip>=0);
	if (iSkip >= 0)
		m_sldSkip.SetPos((int)floor(0.5 + 40.0 * log10((double)iSkip)));
	
	UpdateSkipLabel();
}
