// MainFrame.cpp


#include "stdafx.h"
#include "SimuRoute.h"

#include "MainFrame.h"
#include "DlgProperties.h"
#include "GraphView.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#endif


IMPLEMENT_DYNAMIC(CMainFrame, CMDIFrameWnd)
BEGIN_MESSAGE_MAP(CMainFrame, CMDIFrameWnd)
	ON_WM_CREATE()
	ON_WM_CLOSE()
	ON_COMMAND          (ID_VIEW_PROPERTIES, OnViewProperties)
	ON_UPDATE_COMMAND_UI(ID_VIEW_PROPERTIES, OnUpdateViewProperties)
	ON_UPDATE_COMMAND_UI(ID_INDICATOR_TOTAL, OnUpdateIndicatorTotal)
	ON_UPDATE_COMMAND_UI(ID_INDICATOR_CELL,  OnUpdateIndicatorCoord)
	ON_UPDATE_COMMAND_UI(ID_INDICATOR_UNIT,  OnUpdateIndicatorCoord)
	ON_UPDATE_COMMAND_UI(ID_INDICATOR_STAT,  OnUpdateIndicatorStat)
END_MESSAGE_MAP()


static const UINT s_aIndic[] =
{
	ID_SEPARATOR,
	ID_INDICATOR_TOTAL,
	ID_INDICATOR_CELL,
	ID_INDICATOR_UNIT,
	ID_INDICATOR_STAT,
	ID_INDICATOR_CAPS,
	ID_INDICATOR_NUM,
};


// Creation
int CMainFrame::OnCreate(CREATESTRUCT* pcs)
{
	if (CMDIFrameWnd::OnCreate(pcs)==-1)
		return -1;
	
	// Create toolbar
	if (!m_wndToolBar.CreateEx(this, TBSTYLE_FLAT, WS_CHILD | WS_VISIBLE | CBRS_TOP
		| CBRS_TOOLTIPS | CBRS_FLYBY | CBRS_SIZE_DYNAMIC) ||
		!m_wndToolBar.LoadToolBar(IDR_MAINFRAME))
		return -1;
	
	// Create status bar
	if (!m_wndStatusBar.Create(this) ||
		!m_wndStatusBar.SetIndicators(s_aIndic, nbArray(s_aIndic)))
		return -1;
	
	// Create rebar
	if (!m_wndReBar.Create(this))
		return -1;
	e_dlgRebar.Create(this, IDD_REBAR, CBRS_ALIGN_TOP, AFX_IDW_DIALOGBAR);
	e_dlgRebar.Init();
	m_wndReBar.AddBar(&e_dlgRebar, (LPCTSTR)NULL, NULL, 0);
	
	return 0;
}


// Closing
void CMainFrame::OnClose()
{
	e_dlgProp.SavePlacement();
	
	CMDIFrameWnd::OnClose();
}


// Properties
void CMainFrame::OnViewProperties()
{
	e_dlgProp.ShowWindow((e_dlgProp.IsWindowVisible()) ? SW_HIDE : SW_SHOW);
}

// Properties - Update
void CMainFrame::OnUpdateViewProperties(CCmdUI* pCmdUI)
{
	if (::IsWindow(e_dlgProp))
		pCmdUI->SetCheck(e_dlgProp.IsWindowVisible());
	else
		pCmdUI->Enable(FALSE);
}


// Global information - Update
void CMainFrame::OnUpdateIndicatorTotal(CCmdUI* pCmdUI)
{
	const CGraphView *const pView = app.GetActiveView();
	
	CString s;
	
	if (pView && pView->m_fSimul) {
		int time = pView->m_stats.time;
		if (!time)
			time++;
		
#ifdef PERSO
		s.Format(_T("%d / %d"),
			pView->GetDocument()->GetCell(perso_ptX1).pNode->m_nd.m_delaySynchro,
			perso_delaySynchroStop);
#else
		s.Format(pCmdUI->m_nID,
			pView->m_stats.time,
			pView->m_nbCar,
			(float)pView->m_stats.nbCarCreated / (float)time,
			pView->m_stats.avgCarCreated,
			(float)pView->m_stats.nbCarRefused / (float)time,
			pView->m_stats.avgCarRefused,
			(pView->m_stats.nbCarDestroyed)
			? (float)pView->m_stats.timeCarLive / (float)pView->m_stats.nbCarDestroyed
			: 0.0f,
			pView->m_stats.avgCarLive);
#endif
	}
	
	pCmdUI->SetText(s);
}

// Cell/unit coordinates - Update
void CMainFrame::OnUpdateIndicatorCoord(CCmdUI* pCmdUI)
{
	const CGraphView *const pView = app.GetActiveView();
	
	CPoint pt;
	
	CString s;
	
	// Convert to cell or unit coord
	CPoint ptCur;
	if (pView) {
		GetCursorPos(&ptCur);
		pt = ptCur;
		pView->ScreenToClient(&pt);
		pt += pView->GetScrollPosition();
		
		if (pView->GetViewRect().PtInRect(pt)) {
			// Point is OK
			
			if (pCmdUI->m_nID==ID_INDICATOR_CELL)
				// Cell
				
				pView->ToCell(pt);
				
			else if (pCmdUI->m_nID==ID_INDICATOR_UNIT) {
				// Unit
				
				pt.x /= pView->m_cUnit;
				pt.y /= pView->m_cUnit;
			}
			
			s.Format(pCmdUI->m_nID, pt);
		}
	}
	
	pCmdUI->SetText(s);
}

// Statistics - Update
void CMainFrame::OnUpdateIndicatorStat(CCmdUI* pCmdUI)
{
	const CGraphView *const pView = app.GetActiveView();
	
	CString s;
	
	if (pView && pView->m_fSimul) {
		
		CRect rcLimit;
		rcLimit.SetRectEmpty();
		
		// Get pixel coord
		CPoint pt;
		GetCursorPos(&pt);
		
		pView->ScreenToClient(&pt);
		pt += pView->GetScrollPosition();
		pt.x /= pView->m_cUnit;
		pt.y /= pView->m_cUnit;
		rcLimit = CRect(CPoint(0,0), pView->m_sizeUnit);
		
		if (rcLimit.PtInRect(pt) && pView->m_aUnit) {
			const UNIT &unit = pView->GetUnit(pt);
			if (unit.pGen)
				s.Format(pCmdUI->m_nID, unit.pGen->nbCreated, unit.pGen->nbDestroyed);
		}
	}
	
	pCmdUI->SetText(s);
}
