// Simul.cpp


#include "stdafx.h"

#ifndef _DEBUG
#pragma optimize("t", on)
#endif

#include "Simul.h"

#include "Doc.h"
#include "GraphView.h"

#include "SimuRoute.h"


#ifdef _DEBUG
#define new DEBUG_NEW
#endif


#ifdef MORE_VERIF

void Verif(CGraphView* pView, CAR* aCar[])
{
	for (int x=0; x<pView->m_sizeUnit.cx; x++)
		for (int y=0; y<pView->m_sizeUnit.cy; y++) {
			const CPoint pt(x,y);
			const CAR *const pCar = pView->GetCar(aCar, pt);
			if (pCar && pCar->pt!=pt) {
				const CAR *const pCar2 = pView->GetCar(aCar, pCar->pt);
				x=x;
			}
		}
}

#endif



//*** CGraphView


// Conversion
void CGraphView::ToUnit(CPoint& pt, int iDiagDir) const
{
	ToUnit(pt);
	pt += aoffDiagDir[RoundOrient(iDiagDir)];
}

// Conversion
CPoint CGraphView::GetToUnit(const CPoint pt, int iDiagDir) const
{
	return GetToUnit(pt) + aoffDiagDir[RoundOrient(iDiagDir)];
}


// Move a car to a given position
void CGraphView::SetCar(CAR* apCar[], const CPoint& pt, CAR* pCar)
{
	apCar[pt.x + pt.y*m_sizeUnit.cx] = pCar;
	
	if (pCar) {
		pCar->fCanMove = false;
		pCar->nbMove++;
		pCar->dirTo    = -1;
		pCar->pt       = pt;
	}
}


// Add a new light
CLight& CGraphView::AddLight(const NODEDATA& nd,
	POINT ptDraw, int dirDraw, bool fGreen)
{
	CLight &light = m_lstLight.GetAt(m_lstLight.AddTail(CLight()));
	
	light.m_ptDraw      = ptDraw;
	light.m_dirDraw     = dirDraw;
	light.m_fGreen      = fGreen;
	light.m_delayChange = nd.m_delaySynchro;
	if (fGreen) {
		light.m_timeG     = nd.m_timeG;
		light.m_timeR     = nd.m_timeR;
	}else{
		light.m_timeG     = nd.m_timeR;
		light.m_timeR     = nd.m_timeG;
	}
	
	return light;
}



// Convert from type A to type B
//
// 7  0  4       7  0  1
//                      
// 3     1       6     2
//                      
// 6  2  5       5  4  3
//
// Type A        Type B
//
static const int s_aToClockDir[8] =
{
	0, 2, 4, 6, 1, 3, 5, 7
};
static const int s_aFromClockDir[8] =
{
	0, 4, 1, 5, 2, 6, 3, 7
};


// Register a unit
void CGraphView::SetUnit(CPoint pt, int maxSpeed, CLight* pLight,
	int dir0, int proba0,
	int dir1, int proba1,
	int dir2, int proba2)
{
	UNIT &unit = GetUnit(pt);
	ASSERT(!unit.aMoveDir[0].nbMove);
	
	unit.fValid   = true;
	unit.maxSpeed = maxSpeed;
	unit.pLight   = pLight;
	m_nbUnitValid++;
	
	const MOVE aMove[] =
	{
		{ dir0, proba0 },
		{ dir1, proba1 },
		{ dir2, proba2 },
	};
	
	// Count the valid moves
	int nbMove = 0;
	while (nbMove<nbArray(aMove) && aMove[nbMove].dir>=0)
		nbMove++;
	
	
	for (int iDir=0; iDir<8; iDir++) {
		MOVE_DIR &md = unit.aMoveDir[iDir];
		
		// - Select the compatible directions
		// - Compute the associated total probability
		md.nbMove = 0;
		int probaTotal = 0;
		for (int iMove=0; iMove<nbMove; iMove++) {
			const int diff = abs(s_aToClockDir[iDir] - s_aToClockDir[aMove[iMove].dir]);
			if (diff<=2 || diff>=6) {
				md.aMove[md.nbMove++].dir = aMove[iMove].dir;
				probaTotal += aMove[iMove].probaCumul;
			}
		}
		
		// Compute the cumulated probabilities in %
		int cumul = 0;
		for (int iMove=0; iMove<md.nbMove; iMove++) {
			cumul += aMove[iMove].probaCumul;
			md.aMove[iMove].probaCumul = (probaTotal)
				? cumul * 100 / probaTotal
				: 0;
		}
	}
}


// Registers 4 units for a line road part
// The light is ONLY for THE unit which is an output in the OPPOSITE
// of the given direction
void CGraphView::SetFourUnits(CPoint ptCell, int iDir, int maxSpeed, CLight* pLight)
{
	ToUnit(ptCell);
	
	for (int iDiagDir=0; iDiagDir<4; iDiagDir++)
		SetUnit(ptCell+aoffDiagDir[RoundOrient(iDir + iDiagDir)],
			maxSpeed,
			(iDiagDir==2) ? pLight : NULL,
			RoundOrient(iDir + ((iDiagDir<2) ? 0 : 2)), 1);
}



// Start the simulation mode
void CGraphView::SimulUpdate(bool fOnlyDraw)
{
	if (!m_fSimul)
		return;
	
	if (!fOnlyDraw) {
		
		CDoc *const pDoc = GetDocument();
		
		ZeroMemory(m_aUnit, m_sizeUnit.cx*m_sizeUnit.cy * sizeof(m_aUnit[0]));
		m_lstGen.RemoveAll();
		m_lstLight.RemoveAll();
		m_idCar       = 0;
		m_nbUnitValid = 0;
		ZeroMemory(&m_stats, sizeof(m_stats));
		
		// Store the (x,y) coordinates
		for (int x=0; x<m_sizeUnit.cx; x++)
			for (int y=0; y<m_sizeUnit.cy; y++)
				GetUnit(x,y).pt.SetPoint(x,y);
		
		// If there are errors, refuse to run the simulation
		POSITION pos = pDoc->m_lstNode.GetHeadPosition();
		while (pos)
			pDoc->m_lstNode.GetNext(pos)->UpdateUnits(this);
		
		ZeroMemory(m_apCar, m_sizeUnit.cx*m_sizeUnit.cy * sizeof(m_apCar[0]));
		
		delete [] m_aCar;
		m_aCar        = new CAR[m_nbUnitValid];
		m_nbCar       = 0;
		m_nbWantSnail = 0;
		
		// Search the valid units
		delete [] m_apUnitValid;
		m_apUnitValid = new UNIT*[m_nbUnitValid];
		for (int x=0, iValid=0; x<m_sizeUnit.cx; x++) {
			for (int y=0; y<m_sizeUnit.cy; y++) {
				UNIT &unit = GetUnit(x,y);
				if (unit.fValid)
					m_apUnitValid[iValid++] = &unit;
			}
		}
	}
	
	// Draw the roads once
	DrawRoads(m_road.m_dc, true);
}


// Next tick
void CGraphView::SimulTick()
{
	POSITION pos;
	
	
	// Advance the lights
	pos = m_lstLight.GetHeadPosition();
	while (pos)
		m_lstLight.GetNext(pos).Tick();
	
	
	// - Compute the maximal speed
	// - Reset move count
	int maxCarSpeed = 1;
	for (int iCar=0; iCar<m_nbCar; iCar++) {
		CAR &car = m_aCar[iCar];
		
		const int iSpeed = car.iSpeed;
		if (maxCarSpeed < iSpeed)
			maxCarSpeed = iSpeed;
		
		car.nbMove = 0;
	}
	
	
	for (int iIterSpeed=0; iIterSpeed<maxCarSpeed; iIterSpeed++) {
		
		// Reset m_aCarNew and the MOVING structures
		ZeroMemory(m_apCarNew, m_sizeUnit.cx*m_sizeUnit.cy * sizeof(m_apCarNew[0]));
		for (int x=0; x<m_sizeUnit.cx; x++)
			for (int y=0; y<m_sizeUnit.cy; y++)
				ZeroMemory(&GetUnit(x,y).mov, sizeof(GetUnit(x,y).mov));
		
		// For each car, see if it can move
		// - Maximum speed
		// - Light color
		// - Random break
		for (int iCar=0; iCar<m_nbCar; iCar++) {
			CAR &car = m_aCar[iCar];
			
			const UNIT &unit = GetUnit(car.pt);
			
			// Random brake
			if (car.iSpeed>0 && (rand() % 100) < car.probaBrake)
				car.iSpeed--;
			
			car.fCanMove =
				(car.iSpeed > iIterSpeed) &&
				(unit.maxSpeed >= iIterSpeed) &&
				(!unit.pLight || unit.pLight->m_fGreen);
			
			if (!car.fCanMove && car.timeCreated)
				SetCarPointer(m_apCarNew, car.pt, &car);
		}
		
		CList<CPoint> lstEmpty;
		int nbWantMove = 0;
		
		// - Choose a destination for each car
		// - Delete the cars which are in a pit
		// - List the empty unites (ready to accept a car) to lstEmpty
		// - Compute the maximum car.iSpeed in maxCarSpeed,
		//   it will be used for the next iteration
		for (int iValid=0; iValid<m_nbUnitValid; iValid++) {
			UNIT &unit = *m_apUnitValid[iValid];
			
			CPoint pt = unit.pt;
			CAR *pCar = GetCar(m_apCar, pt);
			ASSERT(!pCar || pCar->pt==pt);
			
			// ZeroMemory() has done it for us
			// unit.posEmpty = NULL;
			
			// If we are in a pit: consider the car as deleted
			if (pCar && unit.aMoveDir[pCar->dir].nbMove==0) {
				if (pCar->timeCreated) {
					ASSERT(unit.pGen);
					if (unit.pGen)
						unit.pGen->nbDestroyed++;
					
					if (pCar->pGen->pNode->m_nd.m_fStats) {
						// Update the statistics
						m_stats.nbCarDestroyed++;
						m_stats.timeCarLive += m_stats.time - pCar->timeCreated;
					}
					
					if (pCar->maxSpeed==maxSpeedSnail)
						m_stats.nbSnail--;
				}
				
				pCar->fCanMove = false;
				pCar->timeCreated = 0;
			}
			
			if (!pCar) {
				// Empty unit
				unit.mov.posEmpty = lstEmpty.AddTail(pt);
				continue;
			}
			
			if (!pCar->fCanMove)
				continue;
			
			
			// Take a moving direction
			if (pCar->dirTo < 0) {
				MOVE_DIR &md = unit.aMoveDir[pCar->dir];
				
				if (md.nbMove==1)
					// Optimize: if there is only one direction, take it
					pCar->dirTo = md.aMove[0].dir;
				
				else{
					// Take a random direction
					
					const int proba = rand() % 100;
					
					for (int i=0; i<md.nbMove; i++) {
						if (proba < md.aMove[i].probaCumul) {
							pCar->dirTo = md.aMove[i].dir;
							break;
						}
					}
				}
			}
			
			if (pCar->dirTo < 0)
				continue;
			
			CPoint ptNext = pt;
			ptNext += aoffDir[pCar->dirTo];
			MOVING &movNext = GetUnit(ptNext).mov;
			if (movNext.fHasMoved)
				continue;
			
			CAR *const pCarNext = GetCar(m_apCar, ptNext);
			if (pCarNext && pCarNext->fCanMove && pCarNext->dirTo>=0 &&
					pCar->dirTo==RoundOrient(pCarNext->dirTo+2)) {   // Opposite directions?
				// We can do a swap: two cars want to go to the another one position,
				// so their direction are opposed
				
				// Swap the cars
				ASSERT(GetCar(m_apCar, ptNext)->timeCreated);
				ASSERT(pCar->timeCreated);
				CAR *const pCarNext = GetCar(m_apCar, ptNext);
				SetCar(m_apCarNew, pt,     pCarNext);
				SetCar(m_apCarNew, ptNext, pCar);
				pCar    ->SetDir(ptNext);
				pCarNext->SetDir(pt);
				SetCar(m_apCar,    pt,     NULL);
				SetCar(m_apCar,    ptNext, NULL);
				
				// Unregister the "want moves"
				nbWantMove -= movNext.nbFrom;
				movNext .nbFrom    = 0;
				unit.mov.nbFrom    = 0;
				movNext .fHasMoved = true;
				unit.mov.fHasMoved = true;
				continue;
			}
			
			
			// Register the car destination (we do a half-turn)
			movNext.aFrom[movNext.nbFrom++] =
				s_aFromClockDir[(s_aToClockDir[pCar->dirTo] + 4) % 8];
			nbWantMove++;
		}
		
		while (lstEmpty.GetCount()>0 && nbWantMove>0) {
			bool fChange = false;
			
			// Try to solve at least one "want move"
			POSITION pos = lstEmpty.GetHeadPosition();
			while (pos && nbWantMove>0) {
				CPoint pt = lstEmpty.GetNext(pos);
				UNIT &unit = GetUnit(pt);
				MOVING &mov = unit.mov;
				
				
				while (mov.nbFrom > 0) {
					// Choose a random source
					
					const int iFrom = rand() % mov.nbFrom;
					const CPoint ptFrom = pt + aoffDir[mov.aFrom[iFrom]];
					MOVING &movFrom = GetUnit(ptFrom).mov;
					
					CAR *const pCarFrom = GetCar(m_apCar, ptFrom);
					if (!pCarFrom) {
						mov.aFrom[iFrom] = mov.aFrom[--mov.nbFrom];
						continue;
					}
					
					// Do the move
					ASSERT(pCarFrom->fCanMove && pCarFrom->timeCreated);
					SetCar(m_apCarNew, pt, pCarFrom);
					pCarFrom->SetDir(ptFrom);
					SetCar(m_apCar, ptFrom, NULL);
					movFrom.fHasMoved = true;
					
					// Register the new empty unit (the source)
					if (movFrom.nbFrom > 0) {
						ASSERT(!movFrom.posEmpty);
						movFrom.posEmpty = lstEmpty.AddHead(ptFrom);
					}
					
					fChange = true;
					nbWantMove -= mov.nbFrom;
					mov.nbFrom  = 0;
					break;
				}
				
				
				// The empty unit is useless: no car want to go there,
				// or the move has been done already
				lstEmpty.RemoveAt(mov.posEmpty);
			}
			
			
			if (!fChange)
				// No change: abort to avoid infinite loop
				break;
		}
		
		
		// The cars which cannot move remain where they are
		for (int iCar=0; iCar<m_nbCar; iCar++) {
			CAR &car = m_aCar[iCar];
			if (car.fCanMove) {
				// Car remained unchanged
				
				SetCarPointer(m_apCarNew, car.pt, &car);
				if (!car.delayChangeDir) {
					car.dirTo = -1;
					car.delayChangeDir = delayCarChangeDir;
				}else
					car.delayChangeDir--;
			}
		}
		
		
		// Swap m_apCar and m_apCarNew
		CAR **const apCarCopy = m_apCar;
		m_apCar    = m_apCarNew;
		m_apCarNew = apCarCopy;
	}
	
	
	// Delete the cars having timeCreated==0
	for (int iCar=0; iCar<m_nbCar; iCar++) {
		CAR &car = m_aCar[iCar];
		
		// car.timeCreated==0 -> car deleted
		while (!car.timeCreated && iCar<m_nbCar) {
			SetCarPointer(m_apCar, car.pt, NULL);
			m_nbCar--;
			if (iCar==m_nbCar)
				break;
			
			car = m_aCar[m_nbCar];
			SetCarPointer(m_apCar, car.pt, &car);
		}
	}
	
	
	// Update the speeds
	for (int iCar=0; iCar<m_nbCar; iCar++) {
		CAR &car = m_aCar[iCar];
		
		if (car.nbMove==car.iSpeed)
			// We could go faster
			car.iSpeed++;
		else if (car.nbMove < car.iSpeed)
			// We can't go at our maximum speed
			car.iSpeed--;
		
		// Car speed limit
		car.OnUpdateSpeed(this);
	}
	
	
	// Advance time
	m_stats.time++;
	
	
	// Create cars with generators
	pos = m_lstGen.GetHeadPosition();
	while (pos) {
		GENERATOR &gen = m_lstGen.GetNext(pos);
		
		if ((rand() % 100) < gen.probaNewCar) {
			// Generate a car
			
			if (GetCar(m_apCar, gen.pt)) {
				// There is already a car
				
				if (gen.pNode->m_nd.m_fStats)
					m_stats.nbCarRefused++;
				continue;
			}
			
			gen.nbCreated++;
			if (gen.pNode->m_nd.m_fStats)
				m_stats.nbCarCreated++;
			
			const bool fSnail = m_nbWantSnail>0;
			if (fSnail) {
				m_nbWantSnail--;
				m_stats.nbSnail++;
				m_stats.nbSnailCreated++;
			}
			
			CAR &car = m_aCar[m_nbCar++];
			
			car.id             = ++m_idCar;
			car.timeCreated    = m_stats.time;
			car.pGen           = &gen;
			car.iSpeed         = 1;
			car.maxSpeed       = (fSnail) ? maxSpeedSnail   : maxSpeedNormal;
			car.probaBrake     = (fSnail) ? probaBrakeSnail : probaBrakeNormal;
			car.dir            = gen.dir;
			car.delayChangeDir = delayCarChangeDir;
			
			SetCar(m_apCar, gen.pt, &car);
		}
	}
}



//*** CLight

// Advance the light
void CLight::Tick()
{
	if (m_delayChange)
		m_delayChange--;
	else{
		m_fGreen      = !m_fGreen;
		m_delayChange = (m_fGreen) ? m_timeG : m_timeR;
	}
}


//*** CAR

static const int aSizeToDir[] =
{
	7,0,4,
	3,0,1,
	6,2,5,
};


// Compute CAR::dir with CAR::pt and ptFrom
void CAR::SetDir(CPoint ptFrom)
{
	const CSize offMove = this->pt - ptFrom;
	
	// dir = aSizeToDir[(offMove.cx+1) + 3*(aSizeToDir.cy+1)];
	dir = aSizeToDir[offMove.cx + 3*offMove.cy + 4];
}


// Validate the speed
void CAR::OnUpdateSpeed(CGraphView* pView)
{
	if (iSpeed < 0) {
		iSpeed = 0;
		return;
	}
	
	const int maxSpeed = min(pView->GetUnit(this->pt).maxSpeed, this->maxSpeed);
	iSpeed = min(iSpeed, maxSpeed);
}
